@@ -1,10 +1,17 @@
 from __future__ import annotations
 
-import warnings
-
 import numpy as np
+import scipy.sparse as sp
 
-from cvxpy.atoms import EXP_ATOMS, NONPOS_ATOMS, PSD_ATOMS, SOC_ATOMS
+import cvxpy.settings as s
+from cvxpy.atoms import (
+    EXP_ATOMS,
+    NONPOS_ATOMS,
+    POWCONE_ATOMS,
+    POWCONE_ND_ATOMS,
+    PSD_ATOMS,
+    SOC_ATOMS,
+)
 from cvxpy.constraints import (
     PSD,
     SOC,
@@ -15,6 +22,7 @@ from cvxpy.constraints import (
     NonNeg,
     NonPos,
     PowCone3D,
+    PowConeND,
     Zero,
 )
 from cvxpy.constraints.exponential import OpRelEntrConeQuad, RelEntrConeQuad
@@ -38,17 +46,15 @@ from cvxpy.reductions.discrete2mixedint.valinvec2mixedint import (
 )
 from cvxpy.reductions.eval_params import EvalParams
 from cvxpy.reductions.flip_objective import FlipObjective
-from cvxpy.reductions.qp2quad_form import qp2symbolic_qp
-from cvxpy.reductions.qp2quad_form.qp_matrix_stuffing import QpMatrixStuffing
 from cvxpy.reductions.reduction import Reduction
 from cvxpy.reductions.solvers import defines as slv_def
 from cvxpy.reductions.solvers.constant_solver import ConstantSolver
 from cvxpy.reductions.solvers.solver import Solver
-from cvxpy.settings import (
-    CLARABEL,
-    PARAM_THRESHOLD,
-)
+from cvxpy.settings import CLARABEL, COO_CANON_BACKEND, DPP_PARAM_THRESHOLD
+from cvxpy.utilities import scopes
 from cvxpy.utilities.debug_tools import build_non_disciplined_error_msg
+from cvxpy.utilities.solver_context import SolverInfo
+from cvxpy.utilities.warn import warn
 
 DPP_ERROR_MSG = (
     "You are solving a parameterized problem that is not DPP. "
@@ -92,17 +98,23 @@ def _is_lp(self):
     return (self.is_dcp() and self.objective.args[0].is_pwl())
 
 
-def _solve_as_qp(problem, candidates):
+def _solve_as_qp(problem, candidates, ignore_dpp: bool = False):
     if _is_lp(problem) and \
             [s for s in candidates['conic_solvers'] if s not in candidates['qp_solvers']]:
         # OSQP can take many iterations for LPs; use a conic solver instead
         # GUROBI and CPLEX QP/LP interfaces are more efficient
         #   -> Use them instead of conic if applicable.
         return False
-    return candidates['qp_solvers'] and qp2symbolic_qp.accepts(problem)
+    # For DPP problems with parameters, check is_qp in DPP scope
+    # because canonicalization will preserve parameters as non-constant
+    if not ignore_dpp and problem.parameters() and problem.is_dpp():
+        with scopes.dpp_scope():
+            return candidates['qp_solvers'] and problem.is_qp()
+    return candidates['qp_solvers'] and problem.is_qp()
 
 
-def _reductions_for_problem_class(problem, candidates, gp: bool = False, solver_opts=None) \
+def _reductions_for_problem_class(problem, candidates, gp: bool = False,
+                                   ignore_dpp: bool = False, solver_opts=None) \
         -> list[Reduction]:
     """
     Builds a chain that rewrites a problem into an intermediate
@@ -118,6 +130,8 @@ def _reductions_for_problem_class(problem, candidates, gp: bool = False, solver_
     gp : bool
         If True, the problem is parsed as a Disciplined Geometric Program
         instead of as a Disciplined Convex Program.
+    ignore_dpp : bool
+        If True, DPP analysis is skipped when checking problem type.
     Returns
     -------
     list of Reduction objects
@@ -157,7 +171,7 @@ def _reductions_for_problem_class(problem, candidates, gp: bool = False, solver_
                        "Consider calling solve() with `qcp=True`.")
         raise DGPError("Problem does not follow DGP rules." + append)
 
-    # Dcp2Cone and Qp2SymbolicQp require problems to minimize their objectives.
+    # Dcp2Cone requires problems to minimize their objectives.
     if type(problem.objective) == Maximize:
         reductions += [FlipObjective()]
 
@@ -168,7 +182,7 @@ def _reductions_for_problem_class(problem, candidates, gp: bool = False, solver_
         reductions += [Valinvec2mixedint()]
 
     use_quad = True if solver_opts is None else solver_opts.get('use_quad_obj', True)
-    valid_qp = _solve_as_qp(problem, candidates) and use_quad
+    valid_qp = _solve_as_qp(problem, candidates, ignore_dpp) and use_quad
     valid_conic = len(candidates['conic_solvers']) > 0
     if not valid_qp and not valid_conic:
         raise SolverError("Problem could not be reduced to a QP, and no "
@@ -230,7 +244,8 @@ def construct_solving_chain(problem, candidates,
     """
     if len(problem.variables()) == 0:
         return SolvingChain(reductions=[ConstantSolver()])
-    reductions = _reductions_for_problem_class(problem, candidates, gp, solver_opts)
+    reductions = _reductions_for_problem_class(problem, candidates, gp, ignore_dpp,
+                                               solver_opts)
 
     # Process DPP status of the problem.
     dpp_context = 'dcp' if not gp else 'dgp'
@@ -239,33 +254,29 @@ def construct_solving_chain(problem, candidates,
         if ignore_dpp:
             reductions = [EvalParams()] + reductions
         elif not enforce_dpp:
-            warnings.warn(DPP_ERROR_MSG)
+            warn(DPP_ERROR_MSG)
             reductions = [EvalParams()] + reductions
         else:
             raise DPPError(DPP_ERROR_MSG)
-    elif any(param.is_complex() for param in problem.parameters()):
-        reductions = [EvalParams()] + reductions
-    else:  # Compilation with DPP.
-        n_parameters = sum(np.prod(param.shape) for param in problem.parameters())
-        if n_parameters >= PARAM_THRESHOLD:
-            warnings.warn(
-                "Your problem has too many parameters for efficient DPP "
-                "compilation. We suggest setting 'ignore_dpp = True'."
-            )
+    else:
+        # Compilation with DPP - auto-select COO backend for large DPP problems
+        if canon_backend is None:
+            total_param_size = sum(p.size for p in problem.parameters())
+            if total_param_size >= DPP_PARAM_THRESHOLD:
+                canon_backend = COO_CANON_BACKEND
 
     # Conclude with matrix stuffing; choose one of the following paths:
-    #   (1) QpMatrixStuffing --> [a QpSolver],
+    #   (1) ConeMatrixStuffing(quad_obj=True) --> [a QpSolver],
     #   (2) ConeMatrixStuffing --> [a ConicSolver]
     use_quad = True if solver_opts is None else solver_opts.get('use_quad_obj', True)
-    if _solve_as_qp(problem, candidates) and use_quad:
-        # Canonicalize as a QP
+    if _solve_as_qp(problem, candidates, ignore_dpp) and use_quad:
+        # Route to a QP solver via the conic canonicalization path
         solver = candidates['qp_solvers'][0]
         solver_instance = slv_def.SOLVER_MAP_QP[solver]
-        # TODO should CvxAttr2Constr come after qp2symbolic_qp?
         reductions += [
+            Dcp2Cone(quad_obj=True),
             CvxAttr2Constr(reduce_bounds=not solver_instance.BOUNDED_VARIABLES),
-            qp2symbolic_qp.Qp2SymbolicQp(),
-            QpMatrixStuffing(canon_backend=canon_backend),
+            ConeMatrixStuffing(quad_obj=True, canon_backend=canon_backend),
             solver_instance,
         ]
         return SolvingChain(reductions=reductions)
@@ -281,14 +292,9 @@ def construct_solving_chain(problem, candidates,
     # the solver will need after canonicalization.
     for c in problem.constraints:
         constr_types.add(type(c))
-    ex_cos = [ct for ct in constr_types if ct in EXOTIC_CONES]
     approx_cos = [ct for ct in constr_types if ct in APPROX_CONES]
     # ^ The way we populate "ex_cos" will need to change if and when
     # we have atoms that require exotic cones.
-    for co in ex_cos:
-        sim_cos = EXOTIC_CONES[co]  # get the set of required simple cones
-        constr_types.update(sim_cos)
-        constr_types.remove(co)
 
     for co in approx_cos:
         app_cos = APPROX_CONES[co]
@@ -300,6 +306,7 @@ def construct_solving_chain(problem, candidates,
     # requiring that cone.
     cones = []
     atoms = problem.atoms()
+
     if SOC in constr_types or any(atom in SOC_ATOMS for atom in atoms):
         cones.append(SOC)
     if ExpCone in constr_types or any(atom in EXP_ATOMS for atom in atoms):
@@ -313,12 +320,10 @@ def construct_solving_chain(problem, candidates,
             or any(atom in PSD_ATOMS for atom in atoms) \
             or any(v.is_psd() or v.is_nsd() for v in problem.variables()):
         cones.append(PSD)
-    if PowCone3D in constr_types:
-        # if we add in atoms that specifically use the 3D power cone
-        # (rather than the ND power cone), then we'll need to check
-        # for those atoms here as well.
+    if PowCone3D in constr_types or any(atom in POWCONE_ATOMS for atom in atoms):
         cones.append(PowCone3D)
-
+    if PowConeND in constr_types or any(atom in POWCONE_ND_ATOMS for atom in atoms):
+        cones.append(PowConeND)
     # Here, we make use of the observation that canonicalization only
     # increases the number of constraints in our problem.
     var_domains = sum([var.domain for var in problem.variables()], start = [])
@@ -336,13 +341,22 @@ def construct_solving_chain(problem, candidates,
             supported_constraints = solver_instance.MI_SUPPORTED_CONSTRAINTS
         else:
             supported_constraints = solver_instance.SUPPORTED_CONSTRAINTS
+
+        solver_context = SolverInfo(solver=solver, supported_constraints=supported_constraints)
+
+        cones = set(cones)
+        ex_cos = (cones & set(EXOTIC_CONES)) - set(supported_constraints)
+
+        for co in ex_cos:
+            sim_cos = set(EXOTIC_CONES[co])
+            cones.update(sim_cos)
+            cones.discard(co)
+
         unsupported_constraints = [
             cone for cone in cones if cone not in supported_constraints
         ]
 
         if has_constr or not solver_instance.REQUIRES_CONSTR:
-            if ex_cos:
-                reductions.append(Exotic2Common())
             if RelEntrConeQuad in approx_cos or OpRelEntrConeQuad in approx_cos:
                 reductions.append(QuadApprox())
 
@@ -353,10 +367,14 @@ def construct_solving_chain(problem, candidates,
                 use_quad_obj = solver_opts.get("use_quad_obj", True)
             quad_obj = use_quad_obj and solver_instance.supports_quad_obj() and \
                 problem.objective.expr.has_quadratic_term()
-            reductions += [
-                Dcp2Cone(quad_obj=quad_obj),
+            reductions.append(
+                Dcp2Cone(quad_obj=quad_obj, solver_context=solver_context),
+            )
+            if ex_cos:
+                reductions.append(Exotic2Common())
+            reductions.append(
                 CvxAttr2Constr(reduce_bounds=not solver_instance.BOUNDED_VARIABLES),
-            ]
+            )
             if all(c in supported_constraints for c in cones):
                 # Check if solver only supports dim-3 SOC cones
                 if solver_instance.SOC_DIM3_ONLY and SOC in cones:
@@ -367,14 +385,14 @@ def construct_solving_chain(problem, candidates,
                     ConeMatrixStuffing(quad_obj=quad_obj, canon_backend=canon_backend),
                     solver_instance
                 ]
-                return SolvingChain(reductions=reductions)
+                return SolvingChain(reductions=reductions, solver_context=solver_context)
             elif all(c==SOC for c in unsupported_constraints) and PSD in supported_constraints:
                 reductions += [
                     SOC2PSD(),
                     ConeMatrixStuffing(quad_obj=quad_obj, canon_backend=canon_backend),
                     solver_instance
                 ]
-                return SolvingChain(reductions=reductions)
+                return SolvingChain(reductions=reductions, solver_context=solver_context)
 
     raise SolverError("Either candidate conic solvers (%s) do not support the "
                       "cones output by the problem (%s), or there are not "
@@ -383,6 +401,46 @@ def construct_solving_chain(problem, candidates,
                           ", ".join([cone.__name__ for cone in cones])))
 
 
+def _validate_problem_data(data) -> None:
+    """Validate problem data for NaN and Inf values.
+
+    Raises ValueError if:
+    - Any matrix/vector contains NaN
+    - Objective or constraint matrix (not RHS/bounds) contains Inf
+
+    Inf is allowed in constraint RHS (b, G) and bounds since users
+    sometimes use inf for unbounded constraints/variables.
+    """
+    # Skip validation for non-dict data (e.g., ConstantSolver returns Problem)
+    if not isinstance(data, dict):
+        return
+
+    # Keys where Inf is allowed (constraint RHS and variable bounds)
+    inf_allowed_keys = {s.B, s.G, s.LOWER_BOUNDS, s.UPPER_BOUNDS}
+
+    # Keys to check (objective coefficients, constraint matrices, bounds)
+    keys_to_check = [s.P, s.C, s.Q, s.A, s.B, s.F, s.G,
+                     s.LOWER_BOUNDS, s.UPPER_BOUNDS]
+
+    for key in keys_to_check:
+        if key not in data or data[key] is None:
+            continue
+        val = data[key]
+        arr = val.data if sp.issparse(val) else val
+        if key in inf_allowed_keys:
+            if np.any(np.isnan(arr)):
+                raise ValueError(
+                    "Problem data contains NaN. "
+                    "Check your parameter values and constants."
+                )
+        else:
+            if not np.all(np.isfinite(arr)):
+                raise ValueError(
+                    "Problem data contains NaN or Inf. "
+                    "Check your parameter values and constants."
+                )
+
+
 class SolvingChain(Chain):
     """A reduction chain that ends with a solver.
 
@@ -400,12 +458,13 @@ class SolvingChain(Chain):
         The solver, i.e., reductions[-1].
     """
 
-    def __init__(self, problem=None, reductions=None) -> None:
+    def __init__(self, problem=None, reductions=None, solver_context=None) -> None:
         super(SolvingChain, self).__init__(problem=problem,
                                            reductions=reductions)
         if not isinstance(self.reductions[-1], Solver):
             raise ValueError("Solving chains must terminate with a Solver.")
         self.solver = self.reductions[-1]
+        self.solver_context = solver_context
 
     def prepend(self, chain) -> "SolvingChain":
         """
@@ -438,6 +497,12 @@ class SolvingChain(Chain):
             A solution to the problem.
         """
         data, inverse_data = self.apply(problem)
+
+        # We validate the data both in SolvingChain.solve and 
+        # in SolvingChain.solve_via_data. These are the two possible 
+        # entry points for executing the solving chain.
+        _validate_problem_data(data)
+
         solution = self.solver.solve_via_data(data, warm_start,
                                               verbose, solver_opts)
         return self.invert(solution, inverse_data)
@@ -478,5 +543,13 @@ class SolvingChain(Chain):
             The information returned by the solver; this is not necessarily
             a Solution object.
         """
+        if solver_opts is None:
+            solver_opts = {}
+
+        # We validate the data both in SolvingChain.solve and 
+        # in SolvingChain.solve_via_data. These are the two possible 
+        # entry points for executing the solving chain.
+        _validate_problem_data(data)
+
         return self.solver.solve_via_data(data, warm_start, verbose,
                                           solver_opts, problem._solver_cache)
